@@ -2,7 +2,7 @@ package Imager::Color::Table;
 use strict;
 use vars qw($VERSION);
 
-$VERSION = "1.002";
+$VERSION = "1.003";
 
     my %colors;
 
@@ -826,12 +826,23 @@ Imager::Color::Table - built-in Imager color table
 
 This class provides a base color table for use in resolving color names.
 
-The table contains the standard X11 rgb.txt colors.
+The table contains the standard X11 C<rgb.txt> colors.
 
 This table is not included as part of Imager::Color itself since it's
 moderately large.
 
-There is only one method: get($name).
+There is only one method:
+
+=over
+
+=item get
+
+  my @rgb = Imager::Color::Table->get('red')
+    or die "No red found";
+
+Retrieves a color from Imager::Color::Tables built-in color table.
+
+=back
 
 =head1 AUTHOR
 
